<?php

if (!defined('_PS_VERSION_')) {
    exit;
}

class Notipack_Integration extends Module
{
    public function __construct()
    {
        $this->name = 'notipack_integration';
        $this->displayName = 'Notipack Integration';
        $this->description = 'This plugin will automatically add notifications from notipack to your site and send the information needed to create them.';
        $this->author = 'Notipack';
        $this->version = '1.3.8';
        $this->tab = 'front_office_features';
        $this->need_instance = 0;

        $this->ps_versions_compliancy = [
            'min' => '1.6.0.0',
            'max' => _PS_VERSION_,
        ];

        $this->bootstrap = true;
        parent::__construct();
    }

    public function install()
    {
        return parent::install() && $this->registerHook('displayHeader') && $this->registerHook('actionPaymentConfirmation');
    }

    public function uninstall()
    {
        return parent::uninstall() && Configuration::deleteByName('NOTIPACK_API_KEY') && Configuration::deleteByName('NOTIPACK_PIXEL_CODE') && Configuration::updateValue('NOTIPACK_API_ACTIVE', false);
    }

    public function hookDisplayHeader($params) {
        $this->context->smarty->assign([
           'pixel_code' => (string)Configuration::get('NOTIPACK_PIXEL_CODE'),
           'pixel_active' => (bool)Configuration::get('NOTIPACK_API_ACTIVE')
        ]);
        return $this->display(__DIR__, '/views/front/pixel.tpl');
    }

    public function hookActionPaymentConfirmation($params) {
        if (Configuration::get('NOTIPACK_API_ACTIVE')) {
            $apiKeyInfo = $this->getApiKeyInfo(Configuration::get('NOTIPACK_API_KEY'));
            if(!is_bool($apiKeyInfo)) {
                $order = new Order((int)$params['id_order']);
                $address = new Address($order->id_address_invoice);
                $products = $order->getProductsDetail();
                $product = new Product((int)$products[0]['product_id']);
                $img =  Product::getCover($product->id);
                $pn = $product->name[strval(count($product->name))];
                $lr = $product->link_rewrite[strval(count($product->link_rewrite))];
                $name = explode(' ', $address->firstname);
                $languages = Language::getLanguages(true, $this->context->shop->id);
                $product_data = [
                    'thumbnail' => $this->context->link->getImageLink(isset($lr) ? $lr : $pn, (int)$img['id_image'], 'small_default'),
                    'buyer_name' => $name[0],
                    'iso_code' => Country::getIsoById($address->id_country),
                    'city' => $address->city,
                ];
                $x = 1;
                $numberOfLangs = count($languages);
                foreach($languages as $language){
                    if($x == 1){
                        $linkKey = 'product_link';
                        $nameKey = 'product_name';
                    }
                    else{
                        $linkKey = 'product_link_' . $x;
                        $nameKey = 'product_name_' . $x;
                    }
                    $product = new Product($product->id, false, (int)$language['id_lang']);
                    $link = new Link();
                    $url = $link->getProductLink($product);
                    $product_data[$nameKey] = $product->name;
                    if($numberOfLangs < 2){
                        $product_data[$linkKey] = $url;
                    }
                    else{
                        $hasIsoCode=false;
                        foreach($languages as $shopLanguage){
                            if(strpos($url, $base_url.$shopLanguage['iso_code']) !== false){
                                $hasIsoCode = true;
                                break;
                            }
                        }
                        if($x == 1){
                            $languageUrl = $url;
                        }
                        else if(!$hasIsoCode){
                            $languageUrl = substr_replace($url, $language['iso_code'].'/', strlen(Context::getContext()->shop->getBaseURL(true)), 0);
                        }
                        else{
                            $languageUrl = substr_replace($url, $language['iso_code'], strlen(Context::getContext()->shop->getBaseURL(true)), 2);
                        }
                        $product_data[$linkKey] = $languageUrl;
                    }
                    $x++;
                }
                foreach($apiKeyInfo['notification_url'] as $not_url) {
                    if($not_url['type'] === 'LATEST_CONVERSION') {
                        $this->sendOrderData($product_data, $not_url['notification_key']);
                    }
                }
            }
        }
    }

    private function sendOrderData($product_data, $url) 
    {
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => false,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => $product_data,
        ));
        $response = curl_exec($curl);
        curl_close($curl);
    }

    private function getAdditionalContent()
    {
        $out = null;
        $out .= '
                <style>
                   @import url("https://fonts.googleapis.com/css2?family=Quicksand:wght@400;500;700&display=swap");
                   @keyframes npi_initiate_underline {
                       from {
                            background-size: 0% 0.55rem;
                       }
                       to {
                            background-size: 100% 0.55rem;
                       }
                   }
                   div#main {
                        background-color: white !important;
                   }
                   .npi_underline {
                       background-image: linear-gradient(to right, #6b35ff4f, #6b35ff4f);
                       background-repeat: no-repeat;
                       background-position: 0 85%;
                       background-size: 0% 0.55rem;
                       animation-name: npi_initiate_underline;
                       animation-delay: 0.3s;
                       animation-duration: 0.5s;
                       animation-iteration-count: 1;
                       animation-fill-mode: both;
                   }
                   .npi_title{
                       font-family: Quicksand, sans-serif !important;
                       font-size: 2.9rem !important;
                       font-weight: bold !important;
                       margin-bottom: 1rem !important;
                   }
                   .text-danger {
                        color: #dc3545!important;
                    }
                    .d-flex {
                        display: flex!important;
                    }
                    .pt-2 {
                        padding-top: .5rem!important;
                    }
                    .pb-2 {
                        padding-bottom: .5rem!important;
                    }
                    .mt-5 {
                        margin-top: 3rem!important;
                    }
                    .text-muted {
                        color: #6c757d!important;
                    }
                    label.control-label.col-lg-3.required {
                        width: auto;
                    }
                    div#content {
                        padding: 150px 10px 0 250px !important;
                    }
                    form#configuration_form, .module_error.alert.alert-danger {
                        max-width: 500px;
                    }
                </style>
        ';
        $out .= '
                <div>
                   <div class="npi_title d-flex">
                      <h1 class="npi_title">
                         <span class="npi_underline">Integracja</span>
                         <span>Z Notipack.com </span>
                      </h1>
                   </div>
                   <p class="text-muted pt-2 mb-3">Tutaj możesz dokonać pełnej integracji z serwisem <b>Notipack.com!</b> <br>
                      Wystarczy, że po utworzeniu kampanii oraz notyfikacji skopiujesz <b>klucz do integracji</b> oraz wkleisz go
                      poniżej!
                   </p>
                </div>
                ';
        return $out;
    }

    public function getContent()
    {
        $out = null;
        $out .= $this->getAdditionalContent();
        if (Tools::isSubmit('submit'.$this->name)) {
            $apiKey = (string)Tools::getValue('NOTIPACK_API_KEY');
            Configuration::updateValue('NOTIPACK_API_KEY', $apiKey);
            if (!$apiKey || empty($apiKey)) {
                $out .= $this->displayError('Klucz API jest niepoprawny');
                Configuration::updateValue('NOTIPACK_API_ACTIVE', false);
                return $out.$this->displayForm();
            }
            $apiKeyInfo = $this->getApiKeyInfo($apiKey);
            if(!$apiKeyInfo) {
                $out .= $this->displayError('Klucz API jest niepoprawny');
                Configuration::updateValue('NOTIPACK_API_ACTIVE', false);
            } else {
                Configuration::updateValue('NOTIPACK_API_ACTIVE', true);
                Configuration::updateValue('NOTIPACK_PIXEL_CODE', $apiKeyInfo['pixel_code']);
            }
        }
        return $out.$this->displayForm();
    }

    private function getApiKeyInfo($apiKey)
    {
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://app.notipack.com/api',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => false,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => array('unique_id' => $apiKey),
        ));
        $c_res = curl_exec($curl);
        curl_close($curl);
        $response = json_decode($c_res, true);
        if (isset($response['errors'])) {
            return false;
        } else {
            return isset($response['data']) ? $response['data'] : false;
        }
    }

    public function displayForm()
    {
        $fieldsForm[0]['form'] = [
          'input' => [
              [
                  'type' => 'text',
                  'label' => 'Klucz API',
                  'name' => 'NOTIPACK_API_KEY',
                  'size' => 20,
                  'required' => true
              ]
          ],
          'submit' => [
              'title' => 'Save',
              'class' => 'btn btn-default pull-right'
          ]
        ];
        $helper = new HelperForm();

        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        $helper->title = '';
        $helper->show_toolbar = false;
        $helper->toolbar_scroll = false;
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = [
            'save' => [
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                    '&token='.Tools::getAdminTokenLite('AdminModules'),
            ],
            'back' => [
                'desc' => $this->l('Back'),
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules')
            ]
        ];
        $helper->fields_value['NOTIPACK_API_KEY'] = Tools::getValue('NOTIPACK_API_KEY', Configuration::get('NOTIPACK_API_KEY'));
        return $helper->generateForm($fieldsForm);
    }

}
